/*
 getRowObject.p
 
 Purpose:
  Retrieve data from the rowObject temp-table and copy contents into
  our temp-table.
 
 Parameters:
   INPUT-OUPUT
     pcRowIdent - Rowident to start with (when given). Will be populated
                  with last Rowident in batch as output.
   INPUT
     piRowsToReturn
     phSdo - handle of SDO to query
     phTT  - handle of temp-table that will receive results
   
 Notes:
   The procedure 'serverSendRows' resides in data.p. The notes below
   are taken from the comments in the procedure.
   
    Parameters:  
      INPUT  piStartRow     - The RowNum value of the record to start the batch
                              to return.  Typically piStartRow is ? as a flag to 
                              use pcRowIdent instead of piStartRow.
      INPUT  pcRowIdent     - The RowIdent of the first record of the batch to
                              to return.  Can also be "FIRST" or "LAST" to force
                              the retrieval of the first (or last) batch of 
                              RowObject records.
      INPUT  plNext         - True if serverSendRows is to start on the "next"
                              record from what is indicated by piStartRow or
                              piRowIdent.
      INPUT  piRowsToReturn - The number of rows in a batch (0 returns all rows)
      OUTPUT piRowsReturned - The actual number of rows returned. This number
                              will either be the same as piRowsToReturn or
                              less when there are not enough records to fill
                              up the batch.
    
    Notes:       If piStartRow is not 0 or ? then pcRowIdent is ignored.
                 plNext is ignored if pcRowIdent is "FIRST" or "LAST".
                 The most common use of piRowsReturned is to indicate that the
                 entire result list has been returned when it is less than 
                 piRowToReturn.
*/
DEFINE INPUT-OUTPUT PARAMETER pcRowIdent     AS CHARACTER NO-UNDO.
DEFINE INPUT        PARAMETER piRowsToReturn AS INTEGER   NO-UNDO.
DEFINE INPUT        PARAMETER phSdo          AS HANDLE    NO-UNDO.
DEFINE INPUT        PARAMETER phTT           AS HANDLE    NO-UNDO.

DEFINE VARIABLE hRowObj    AS HANDLE   NO-UNDO. /* handle of RowObject temp table */
DEFINE VARIABLE hRowObjQry AS HANDLE   NO-UNDO. /* handle of RowObject temp table query */
DEFINE VARIABLE iRowsRet   AS INTEGER  NO-UNDO.

RUN serverSendRows IN phSdo
  (INPUT ?,           /* ? = use the 2nd parameter as starting point */
   INPUT pcRowIdent,   
   INPUT (IF pcRowident <> "FIRST":U AND pcRowIdent <> "LAST":U THEN
            TRUE
          ELSE
            FALSE),       
   INPUT piRowsToReturn,
   OUTPUT iRowsRet,   
   OUTPUT TABLE-HANDLE phTT).

/*
 If we are batching records, we return the ROWID of the last record in the
 batch to make life easier on the client. This codes checks to see if we got
 less rows than requested.  If so, we can assume we are at the last batch so
 we just return a blank instead of a ROWID.
*/
IF piRowsToReturn GT 0 THEN DO:
  IF iRowsRet LT piRowsToReturn THEN
    pcRowIdent = "".
  ELSE DO:
    phTT:DEFAULT-BUFFER-HANDLE:FIND-LAST().
    pcRowIdent = phTT:DEFAULT-BUFFER-HANDLE:BUFFER-FIELD("rowIdent"):BUFFER-VALUE.
  END.
END.
ELSE
  pcRowIdent = "".
  
RETURN.
