/*
 setRowObject.p
 
 Purpose:
  Submit specified DataObject rowObject changes.
  
 Parameters:
  INPUT
    pcSdo - path to SDO to start
    phTT  - handle to RowObject temp-table to submit
*/
DEFINE INPUT  PARAMETER pcSdo AS CHARACTER NO-UNDO.
DEFINE INPUT  PARAMETER phTT  AS HANDLE    NO-UNDO.

FUNCTION getRowMod RETURNS CHARACTER PRIVATE	(  ) FORWARD.
	
DEFINE VARIABLE cMessages  AS CHARACTER   NO-UNDO.
DEFINE VARIABLE cUndoIds   AS CHARACTER   NO-UNDO.
DEFINE VARIABLE hSdo       AS HANDLE      NO-UNDO.
DEFINE VARIABLE hRowObjUpd AS HANDLE      NO-UNDO.
DEFINE VARIABLE lSuccess   AS LOGICAL     NO-UNDO INITIAL TRUE.
DEFINE VARIABLE lAdd       AS LOGICAL     NO-UNDO INITIAL NO.

$UPDATE-ROW:
DO ON ERROR UNDO, LEAVE:

  RUN create_update_record.

  RUN VALUE(pcSdo) PERSISTENT SET hSdo.
  
  RUN serverCommit IN hSdo(INPUT-OUTPUT TABLE-HANDLE hRowObjUpd, OUTPUT cMessages, OUTPUT cUndoIds).
  
  IF cMessages <> "" THEN
    LEAVE $UPDATE-ROW.

  IF getRowMod() = "A":U THEN
    RUN update_rowident.
    
END. /* $UPDATE-ROW */

/*
 If any update step was not successful, then return ADM2 messages to caller
*/
IF cMessages = "" THEN
  cMessages = DYNAMIC-FUNCTION('fetchMessages':U IN hSdo).

RETURN cMessages.

FINALLY:
  RUN destroyObject IN hSdo.
  DELETE OBJECT hRowObjUpd NO-ERROR.
END FINALLY.


PROCEDURE create_update_record PRIVATE:
  /*
   Create a copy the dataset submitted by the client. The 'changedFields'
   field must be added as the ADM2 framework populates this field with the
   detected changes.
  */
  DEFINE VARIABLE hBuf1 AS HANDLE  NO-UNDO. /* buffer of TT sent my client */
  DEFINE VARIABLE hBuf2 AS HANDLE  NO-UNDO. /* buffer of new RowObjUpd */
  DEFINE VARIABLE hQry  AS HANDLE  NO-UNDO.
  DEFINE VARIABLE lOk   AS LOGICAL NO-UNDO.

  $CREATE-ROWOBJUPD:
  DO ON ERROR UNDO, LEAVE:

    CREATE TEMP-TABLE hRowObjUpd.

    hRowObjUpd:CREATE-LIKE(phTT).
    hRowObjUpd:ADD-NEW-FIELD("changedFields","char").
    hRowObjUpd:TEMP-TABLE-PREPARE("RowObjUpd").

    /*
     Set the buffers, create a query, and copy given dataset..
    */
    hBuf1 = phTT:DEFAULT-BUFFER-HANDLE. 
    hBuf2 = hRowObjUpd:DEFAULT-BUFFER-HANDLE.

    CREATE QUERY hQry.

    hQry:SET-BUFFERS(hBuf1).
    hQry:QUERY-PREPARE("FOR EACH " + phTT:NAME).
    hQry:QUERY-OPEN.

    hQry:GET-FIRST.
    DO WHILE NOT hQry:QUERY-OFF-END:
      hBuf2:BUFFER-CREATE.
      hBuf2:BUFFER-COPY(hBuf1).
      hQry:GET-NEXT.
    END.
  END. /* $ROWOBJUPD */

  IF VALID-HANDLE(hQry) THEN
    DELETE OBJECT hQry.

END PROCEDURE.

PROCEDURE update_rowident PRIVATE:
  /*
   Update the rowident of the given input table
   Notes: Obviously, this will not work if several "A"dd records
   are included in a single temp-table. This assumes that "A"dd
   records will only be given one at a time.
  */
  DEFINE VARIABLE cRowIdent AS CHARACTER NO-UNDO.
  DEFINE VARIABLE hBuf      AS HANDLE    NO-UNDO.
  
  hBuf = hRowObjUpd:DEFAULT-BUFFER-HANDLE.
  hBuf:FIND-FIRST().
  
  IF hBuf:AVAILABLE THEN DO:
    
    cRowIdent = hBuf::rowIdent.
    
    hBuf = phTT:DEFAULT-BUFFER-HANDLE.
    hBuf:FIND-FIRST().
    
    IF hBuf:AVAILABLE THEN
      hBuf::rowIdent = cRowIDent.  
    
  END.

  RETURN.

END PROCEDURE.

FUNCTION getRowMod RETURNS CHARACTER PRIVATE (  ):
  /*
   Fetch the rowMod for the first record of the input table.
   For "U"pdates, there will be two records (with the first
   one having a blank rowMod.  However, we are really only
   interested in the "A"dd records so it's no big deal.
  */
  DEFINE VARIABLE hBuf AS handle NO-UNDO.
  
  hBuf = phTT:default-buffer-handle.
  hBuf:find-first().
  
  IF hBuf:AVAILABLE THEN
    RETURN hBuf::rowMod.
  ELSE  
    RETURN "".
		
END FUNCTION.

